function regionSamAll = breakIntoTiles(imSize, regionChunkSize, regionChunkNums, pad)

% each row describes a tile as
%   [tileRegion paddedTileRegion cropIndices]
% where each of those three are given as
%   [w1 w2 h1 h2]

if (nargin < 3) | isempty(regionChunkNums)
    % break the whole image into tiles
    regionTopLeft = [1 1];
    regionChunkNums = ceil( fliplr(imSize)./regionChunkSize );
end
if (nargin < 4)
    pad = 2;
end

nRegions = 0;
for chunkRow = 1:regionChunkNums(1)
    for chunkCol = 1:regionChunkNums(2)
        rr = regionTopLeft(1) + regionChunkSize(1)*(chunkRow-1);
        rc = regionTopLeft(2) + regionChunkSize(2)*(chunkCol-1);
        regionSam = [rr rr+regionChunkSize(1)-1 rc rc+regionChunkSize(2)-1];
        if (regionSam(1) > imSize(2)) | (regionSam(3) > imSize(1))
            continue;
        end
        nRegions = nRegions + 1;
        regionSam = [max(min(regionSam(1:2),imSize(2)),1) max(min(regionSam(3:4),imSize(1)),1)];
        
        paddedRegion = regionSam + [-pad pad -pad pad];
        paddedRegion = [max(min(paddedRegion(1:2),imSize(2)),1) max(min(paddedRegion(3:4),imSize(1)),1)];
        regionCrop = regionSam - [paddedRegion(1) paddedRegion(1) paddedRegion(3) paddedRegion(3)] + [1 1 1 1];
        
        regionSamAll(nRegions,:) = [regionSam paddedRegion regionCrop];
    end
end
