function doIsocontourJob(bigjobdir,outputdir,tilesToProcess,symmetricToo,radNormalizeType)

if nargin < 4
    symmetricToo = 0;
end
if nargin < 5
    radNormalizeType = 0;
end

cd(bigjobdir);
if isempty(outputdir)
    outputdir = pwd;
end
outputdir = appendSlash(outputdir);


if matlabVersion < 7
    load5('bigjob-isocontour.mat');
else
    load('bigjob-isocontour.mat');
end
% contains:
%   tilesData, isoAssign, isoAssignSym
%   radCorrectFactorOverall 
%     [sometimes, only required if radNormalizeType==2]

for I=1:length(tilesToProcess)
    regionInd = tilesToProcess(I)
    tic;

    if regionInd<1 | regionInd>size(tilesData,1)
        fprintf(1,'bad regionInd %d\n',regionInd);
        continue;
    end
    
    region    = tilesData(regionInd,1:4);
    regionStr = ['aperFocus',sprintf('-%04d',region),'.mat'];
    fprintf(1,'bigjob patch %04d [%d/%d] %s\n',regionInd,I,length(tilesToProcess),regionStr);
    if matlabVersion < 7
        load5(regionStr);
    else
        load(regionStr);
    end
    % contains:
    %   aperFocusImage
    aperFocusImage = single(aperFocusImage);
    szAFI = size(aperFocusImage);

    % radiometrically normalize the AFI, to account for lighting changes
    % at sequence capture time
    if (radNormalizeType==1)
        % do the normalization on the fly, for this patch
        %fprintf(1,'on the fly AFI rad correction\n');
        radCalibFactor = 1./sum(sum(sum(aperFocusImage,5),4),3);
        radCalibFactor = radCalibFactor/max(radCalibFactor(:));
        aperFocusImage = aperFocusImage .* repmat(radCalibFactor,[1 1 szAFI(3:5)]);
    elseif (radNormalizeType==2)
        % global radiometric AFI correction factor provided (in 'bigjob-isocontour.mat')
        aperFocusImage = aperFocusImage .* repmat(radCorrectFactorOverall,[1 1 szAFI(3:5)]);
    end
    
    errConfocal   = metricConfocal(aperFocusImage); 
    %errWideNarrow = metricWideNarrow(aperFocusImage); 
    errIsocontour = metricIsocontour(aperFocusImage, isoAssign); 

    depthConfocal = depthGreedy(errConfocal);
    %sigSpatial =  7.0;  % 1.5
    %sigDepth   = 10.0;
    %[depthConfocalBilat,confidenceConfocalBilat]     = depthBilateralICM(errConfocal,sigSpatial,sigDepth);
    %[depthWideNarrowBilat,confidenceWideNarrowBilat] = depthBilateralICM(errWideNarrow,sigSpatial,sigDepth);  
    depthIsocontour = depthGreedy(errIsocontour);
    
    if symmetricToo
        errIsocontSym = metricIsocontour(aperFocusImage, isoAssignSym); 
        depthIsocontSym = depthGreedy(errIsocontSym);
    end
        
    outputStr = ['isocontour',sprintf('-%04d',region),'.mat'];
    if matlabVersion < 7
        save([outputdir,outputStr],'err*','depth*','region');
        %save([outputdir,outputStr],'err*','depth*','confidence*','region');
    else
        save([outputdir,outputStr],'err*','depth*','region','-v6');
        %save([outputdir,outputStr],'err*','depth*','confidence*','region','-v6');
    end
    
    toc;
end
