%%%%%%%%%%%%%%%%
% in each file
%
% - region: coords of the region, same as in filename, [h1 h2 w1 w2]
% - aperFocusImage: aperture-focus image for the region [numAper=13 numFocus=61 regionHeight regionWidth numColors=3]

%%%%%%%%%%%%%%%%
% some setup code:

S=1; 
sequenceDir{S}='I:\Backup Capture (ECCV 2006)\seq_planthead\';  % your own dir
seqToProcess=1;

cameraName   = 'EOS-1Ds';
sensorWidth  = 35.8;  % in mm
focalLength = 85;
imSize = [4064 2704];  % [w h]
nAper = 13;
nFocalPlanes = 61;
fStops = [1.2 1.4 1.6 1.8 2 2.8 3.5 4.5 5.6 7.1 9 11 16];

targetAperFocusBytes = 75 * 1024^2;  % MB/file
targetWidth = sqrt(targetAperFocusBytes/(nAper*nFocalPlanes*3*2));
regionChunkSize = floor( [targetWidth targetWidth] )
regionChunkNums = [];  % break whole image into tiles
regionPad = 0;
tilesData = breakIntoTiles(imSize, regionChunkSize, regionChunkNums, regionPad);
nTiles = size(tilesData,1);

%%%%%%%%%%%%%%%%%
% global radiometric post-correction, due to capture-time lighting variations etc.

nTiles = size(tilesData,1);
imCent = fliplr(imSize/2);
clear radCorrectFactor numPixelsUnsaturated
numPixelsUnsaturated = zeros(nTiles,1);
for I=1:nTiles
    region=tilesData(I,1:4);
    % if tile is close to the center ..
    if sum(((region([1 3])+region([2 4]))/2 - imCent).^2) < (max(imSize)/10)^2
        tic;
        region
        load([sequenceDir{S},'aperFocus',sprintf('-%04d',region),'.mat']);
        aperFocusImage = single(aperFocusImage);

        % accumulate AFI measurements (A,F) over all pixels (except saturated ones)

        isSaturated = (aperFocusImage >= 65280) | (aperFocusImage <= 1024);
        isSaturated = sum(sum(sum(isSaturated,1),2),5) > 0;
        isSaturatedMask = repmat(isSaturated,[size(aperFocusImage,1) size(aperFocusImage,2) 1 1 size(aperFocusImage,5)]);
        aperFocusImage(isSaturatedMask) = 0;
        radCorrectFactor(:,:,I) = sum(sum(sum( aperFocusImage ,5),4),3);
        numPixelsUnsaturated(I) = size(aperFocusImage,3)*size(aperFocusImage,4) - sum(sum(isSaturated));
        %figure(1); clf; imagesc(squeeze(isSaturated)); colormap(gray);
        %figure(2); clf; imagesc(radCorrectFactor(:,:,I)); colormap(gray); drawnow;
        %figure(205); clf; image(min(single(squeeze(aperFocusImage(end,31,:,:,:)))/65535,1))
        toc;
    else
        continue;
    end
end
radCorrectFactorOverall = zeros(nAper,nFocalPlanes);
for I=1:nTiles
    if (numPixelsUnsaturated(I) > 0)
        radCorrectFactorOverall = radCorrectFactorOverall + radCorrectFactor(:,:,I);
    end
end
radCorrectFactorOverall = 1./radCorrectFactorOverall;
radCorrectFactorOverall = radCorrectFactorOverall/max(radCorrectFactorOverall(:));  % factor should darken
figure(72); clf; imagesc(radCorrectFactorOverall); colormap(gray); drawnow;

%%%%%%%%%%%%%%%%%
% prepare for large job that evaluates the isocontour detector, split over 
% multiple processors

% distance from the lens to extreme focal planes, in mm, from the FARO arm
% sensor-scene distance and the thin lens formula
dNear =  848.2;  % 1200
dFar  = 1013.3;  % 1350

enforceSymmetry = 0;
[isoContrib isoContribInv isoAssign] = ...
    genIsocontourModel(fStops,nFocalPlanes, ...
                       focalLength,dNear,dFar,sensorWidth,imSize, ...
                       enforceSymmetry);
enforceSymmetry = 1;
[isoContribSym isoContribInvSym isoAssignSym] = ...
    genIsocontourModel(fStops,nFocalPlanes, ...
                       focalLength,dNear,dFar,sensorWidth,imSize, ...
                       enforceSymmetry);
for S=seqToProcess
    save([sequenceDir{S},'bigjob-isocontour.mat'],'tilesData','isoAssign','isoAssignSym','radCorrectFactorOverall','-v6');
end

%%%%%%%%%%%%%%%%%
% hacky parallel processing by hand, generate functions to execute

cd(sequenceDir{S})
nProcessors = 10
load('bigjob-isocontour.mat')
nTiles = size(tilesData,1);
for I=1:nTiles
    region=tilesData(I,1:4); 
    fn = dir(['isocontour',sprintf('-%04d',region),'.mat']); 
    fnexist = ~isempty(fn) && fn(1).bytes>0; 
    tileDone(I) = fnexist; 
end
undone = find(~tileDone);
nUndone = length(undone);
processorAlloc = repmat([1:nProcessors],[1 ceil(nUndone/nProcessors)]);
processorAlloc = processorAlloc(1:nUndone)
for I=1:nProcessors
    thisUndone = undone(processorAlloc == I);
    fprintf(1,'doIsocontourJob(''%s'',[],[ ',sequenceDir{S});    
    fprintf(1,'%d ',thisUndone);
    fprintf(1,'],0,2)\n');  % global radiometric AFI correction factor (in 'bigjob-isocontour.mat')
end
doIsocontourJob(sequenceDir{S},[],undone,0,2);  % global radiometric AFI correction factor 

