function [isoContrib, isoContribInv, isoAssign, blurRadii] = ...
    genIsocontourModel(fStops,nFocalPlanes, ...
                       focalLength,dNear,dFar,sensorWidth,imSize, ...
                       enforceSymmetry)

nAper = length(fStops);
if (size(fStops,1) < size(fStops,2)), fStops = fStops'; end   % make a column vector

% corresponding distance to image plane, by thin lens formula
imnear = 1/ (1/focalLength - 1/dNear);
imfar  = 1/ (1/focalLength - 1/dFar);
% assume focus setting moves the image plane linearly
imdists   = linspace(imnear,imfar,nFocalPlanes);

% object distances, by thin lens formula
objdists0 = linspace(dNear,dFar,nFocalPlanes);  % naive, linear in space
objdists  = 1./ (1/focalLength - 1./imdists);             % by thin lens formula
%figure(1); clf; plot(objdists); hold on; plot(objdists0,'r')

% aperture focus image
af_aper     = repmat( fStops, [1 nFocalPlanes] );
af_imdists  = repmat( imdists,  [nAper 1] );
af_objdists = repmat( objdists, [nAper 1] );
af_rlens    = (focalLength./af_aper)/2;

nPixels = nAper*nFocalPlanes;
synClosestAperFocus = repmat(single(0),[nPixels nPixels nFocalPlanes]);
%synClosestAperFocus = zeros([nPixels nPixels nFocalPlanes],'single');
fprintf(1,'isocontour model (symmetry=%d) ',enforceSymmetry);
for indf_true = 1:nFocalPlanes
    fprintf(1,'.');
    
    % radius of blur circle (on the image plane), from geometric optics
    imdist_true = imdists(indf_true);
    objdist_true = objdists(indf_true);
    af_rblur = af_rlens .* abs(af_imdists - imdist_true)./imdist_true;    % old method, amount of blur in the image plane
    %af_rblur = af_rlens .* abs(af_objdists - objdist_true)./objdist_true; % (incorrect!)
    %af_rblur = af_rlens .* abs(objdist_true - af_objdists)./af_objdists;   % new method, amount of blur in the scene
   
    % convert radius of blur circle to pixels
    npixelswidth = imSize(1);
    pixelsize = sensorWidth/npixelswidth;

    af_pixelblur = af_rblur/pixelsize;                             % old method, amount of blur in the image plane!
    %af_pixelblur = af_rblur./af_objdists*imdist_true /pixelsize;    % new method, amount of blur in the scene (correct? why??)
    %figure(2); clf; mesh(af_pixelblur)
    blurRadii(indf_true,:,:) = af_pixelblur;
    
    %af_rblur = af_rlens .* abs(af_imdists - imdist_true)./imdist_true;  % old method, blur in the image plane!
    %qq0 = af_rblur/pixelsize;
    %af_rblur = af_rlens .* abs(af_objdists - objdist_true)./objdist_true;
    %qq1 = af_rblur./af_objdists.*af_imdists /pixelsize;
    %qq2 = af_rblur./af_objdists*imdist_true /pixelsize;
    %norm(qq0-qq1)
    %norm(qq0-qq2)
    %norm(qq1-qq2)
    %max(max(abs(qq1-qq2)./qq1))

    
    % slightly more accurate calculation of DOF ..
    %c = 0.03
    c = 2*(pixelsize/2);  % circle of confusion diameter
    d = objdist_true;
    frontDOF = c*fStops.*d.^2 ./ (focalLength^2 + c*fStops.*d);
    backDOF  = c*fStops.*d.^2 ./ (focalLength^2 - c*fStops.*d);
    sizeDOF = backDOF-frontDOF;
    % find corresponding focus settings for DOF
    indf_frontDOF = interp1(objdists,[1:nFocalPlanes],d-frontDOF);
    indf_backDOF  = interp1(objdists,[1:nFocalPlanes],d+backDOF);
    % clip
    indf_frontDOF( d-frontDOF < objdists(1) )            = 1;
    indf_frontDOF( d-frontDOF > objdists(nFocalPlanes) ) = nFocalPlanes;
    indf_backDOF ( d+backDOF  < objdists(1) )            = 1;
    indf_backDOF ( d+backDOF  > objdists(nFocalPlanes) ) = nFocalPlanes;
    indDOF = round([indf_frontDOF indf_backDOF]);
    % everything in DOF assigned a blur of zero
    af_pixelblur1 = af_pixelblur;
    for I=1:nAper
        af_pixelblur1(I,indDOF(I,1):indDOF(I,2)) = 0;
    end
    %figure(4); clf; mesh(af_pixelblur1)

    % define isocontours regions based on the top row (full aperture) pixels
    % are these regions of "constancy" really comparable across focus settings??

    if enforceSymmetry
        % pick the side with the most pixels to define the isocontours
        if indf_true > nFocalPlanes/2
            toprow = af_pixelblur1(1,indf_true:-1:1);
        else
            toprow = af_pixelblur1(1,indf_true:nFocalPlanes);
        end
    else
        % to break the symmetry, make half the isocontours negative!
        % use all the pixels to define the isocontours
        af_pixelblur1(:,indf_true+1:nFocalPlanes) = -af_pixelblur1(:,indf_true+1:nFocalPlanes);
        toprow = af_pixelblur1(1,:);
    end
    toprow = unique(toprow);
    ncontours = length(toprow);
    % linear interpolation to get boundaries of isocontour regions
    contourBounds = [toprow(1) (toprow(1:ncontours-1) + toprow(2:ncontours))/2 toprow(ncontours)];

    % hard assignment of each pixel to an isocontour
    indHardAssign = floor( interp1(contourBounds,1:ncontours+1,af_pixelblur1) );
    indHardAssign( indHardAssign==ncontours+1 ) = ncontours;

    % soft assignment of "border" pixels to <=2 isocontours, using
    % linear interpolation
    contrib = zeros(nAper,nFocalPlanes,ncontours);
    for I=1:nAper
        % boundaries of this row of pixels
        aprow = af_pixelblur1(I,:);
        rowBounds = [aprow(1) (aprow(1:nFocalPlanes-1) + aprow(2:nFocalPlanes))/2 aprow(nFocalPlanes)];
        rowBounds = min(rowBounds,max(contourBounds));  % tweak: should be clamped
        rowBounds = max(rowBounds,min(contourBounds));

        % what isocontour boundary indices do these boundaries correspond to?
        indBounds = interp1(contourBounds,[1:ncontours+1],rowBounds);
        actually_ints = abs(indBounds - floor(indBounds)) < 1e-10;
        indBounds(actually_ints) = round(indBounds(actually_ints));

        % record contributions of each isocontour to each pixel
        for J=1:nFocalPlanes
            bnds = sort( rowBounds(J:J+1) );
            bnds_interp = interp1(contourBounds,1:ncontours+1,bnds);
            if abs(bnds_interp(2)-bnds_interp(1)) < 1e-10
                bnds_interp(2) = bnds_interp(1);  % tweak
            else
                bnds_interp = bnds_interp + [1 -1]*1e-10;  % tweak  
            end
            contourInd = floor( bnds_interp );
            contourInd = min(contourInd,ncontours);  % tweak: should be clamped
            contourInd = max(contourInd,1);

            if contourInd(1)==contourInd(2)
                contrib(I,J,contourInd(1)) = 1;
            elseif contourInd(1)+1==contourInd(2)
                w = bnds_interp(2)-bnds_interp(1);
                alpha = (contourInd(2)-bnds_interp(1))/w;
                contrib(I,J,contourInd(1)) =   alpha;
                contrib(I,J,contourInd(2)) = 1-alpha;
            else
                I,J,contourInd,bnds_interp-2
                disp('ERROR: isocontour regions should not be narrower than a pixel');
            end
        end

        %norm( sum( squeeze(contrib(I,:,:)), 2) - ones(nFocalPlanes,1) )
        %figure(11); clf; imagesc( squeeze( contrib(I,:,:) ) ); colormap(gray(256)); pause(0.1);
    end
    %for C=1:ncontours, figure(11); clf; imagesc(contrib(:,:,C)); colormap(gray(256)); pause(0.1); end
    %norm( sum(contrib,3) - ones(nAper,nFocalPlanes) )

    contrib0 = reshape(contrib,[nAper*nFocalPlanes ncontours]);
    contrib0_inv = pinv(contrib0);
    contrib0_inv( abs(contrib0_inv) < 1000*eps ) = 0;
   
    isoAssign(:,indf_true) = indHardAssign(:);

    % sparse single arrays are not supported! (doubles takes too much memory)
    %isoContrib{indf_true}    = sparse(contrib0);
    %isoContribInv{indf_true} = sparse(contrib0_inv);
    isoContrib{indf_true}    = contrib0;
    isoContribInv{indf_true} = contrib0_inv;
    
    % too big to fold them together ...
    %synClosestAperFocus(:,:,indf_true) = single( contrib0*contrib0_inv );

    %[ncontours rank(contrib0)]
    %figure(12); clf; imagesc(contrib0); colormap(gray(256));
    %figure(13); clf; imagesc(contrib0_inv); colormap(gray(256));
end
fprintf(1,'\n');

