function err = metricIsocontour(aperFocusImage, isoAssign, onlySomeAper, onlySomeIsocontours)
%function err = metricIsocontour(aperFocusImage, isoContrib, isoContribInv, isoAssign)

% Evaluate the "isocontour" metric for the (calibrated)
% aperture-focus image given.
%
% We predict constant intensity across the depth-of-field. Furthermore 
% all settings with the same corresponding "blur circle size" will
% also have similar intensities. 

if ~isa(aperFocusImage,'single')
    aperFocusImage = single(aperFocusImage);
end    

sz = size(aperFocusImage);
aperFocusImage = reshape(aperFocusImage, [sz(1)*sz(2) sz(3:5)]);
err            = repmat(single(0),sz(2:4));

if nargin>2 & ~isempty(onlySomeAper)
    sz_full = size(aperFocusImage);
    aperFocusImage = aperFocusImage(onlySomeAper,:,:,:,:);
    isoAssign = reshape(isoAssign,[sz_full(1) sz_full(2) size(isoAssign,2)]);
    isoAssign = isoAssign(onlySomeAper,:,:);
    isoAssign = reshape(isoAssign,[length(onlySomeAper)*sz_full(2) size(isoAssign,3)]);
end

% precompute indices corresponding to each isocontour region
if nargin>3 & ~isempty(onlySomeIsocontours)
    % only consider a few isocontours around the depth hypothesis .....
    % don't accumulate error over the whole AFI .. part-way between
    % confocal constancy and full AFI model fitting, so as not to
    % assume fronto-planarity in such a large region .. hopefully it
    % will exaggerate our lack of knowledge about textureless pixels
    % adjacent to bright distinctive pixels
    % 
    
    for f = 1:sz(2)
        f_isoAssign = reshape( isoAssign(:,f), [sz(1:2)] );
        f_indIsocontour = f_isoAssign(1,f);
        cind = 1;
        for c = 1:max(isoAssign(:))
            if (abs(c - f_indIsocontour) <= onlySomeIsocontours)
                indIsocontour{f,cind}   = find( isoAssign(:,f)==c );
                indIsocontourSz(f,cind) = length(indIsocontour{f,cind});
                cind = cind+1;
                %figure(19); clf; imagesc(reshape(isoAssign(:,f)==c,sz(1:2))); axis image; colormap(gray(256)); pause(0.2);
            end
        end     
    end
    nIsocontoursMax = size(indIsocontourSz,2);
    
else
    % all the isocontours
    nIsocontoursMax = max(isoAssign(:));
    for f = 1:sz(2)
        for c = 1:nIsocontoursMax
            indIsocontour{f,c}   = find( isoAssign(:,f)==c );
            indIsocontourSz(f,c) = length(indIsocontour{f,c});
        end
    end    
end
indIsocontourSz(indIsocontourSz==0) = 1;  % to avoid divide by zero

%tic;

% calculate sum-of-squares difference, ie. Var[X]*n(X), for each isocontour 
% and accumulate for each focus hypothesis, and over RGB
aperFocusImage2 = aperFocusImage.^2;
sum1  = repmat(single(0),[sz(3:5)]);
sum2  = repmat(single(0),[sz(3:5)]);
for f = 1:sz(2)
    for c = 1:nIsocontoursMax
        sum1 = sum( aperFocusImage(indIsocontour{f,c},:,:,:), 1);
        sum2 = sum( aperFocusImage2(indIsocontour{f,c},:,:,:), 1);
        err(f,:,:) = err(f,:,:) + sum( sum2 - sum1.^2/indIsocontourSz(f,c), 4 );
    end
end
%toc;

% % evaluate consistency across isocontours defined by the top row 
% % (full-aperture) across different "true" focus settings
% 
% aperFocusImageSyn = repmat(single(0),[sz(1)*sz(2) sz(3:5)]);
% subsetMean        = repmat(single(0),sz(3:5));
% assignInd = repmat(uint8(0),[sz(1)*sz(2) 1]);
% tic;
% nContoursIsoAssign = max(isoAssign);
% for indf_true = 1:sz(2)
%     % fit optimal colors per isocontour, then synthesize new aper-focus image
%     %contour_colors = isoContribInv{indf_true} * imflat;
%     %imflat_syn = isoContrib{indf_true} * contour_colors;
% 
%     % hard assignment of aperture-focus setting to an isocontour, faster 
%     %tic;
%     for I=1:nContoursIsoAssign(indf_true)
%         indI = find( isoAssign(:,indf_true)==I );
%         subsetMean = mean( aperFocusImage(indI,:,:,:), 1 );
%         aperFocusImageSyn(indI,:,:,:) = repmat(subsetMean,[length(indI) 1 1 1]);        
%     end
%     %toc;
% 
%     % consider partial contributions from each isocontour, 2x slower
% %     tic;
% %     aperFocusImageSyn = reshape( ...
% %         isoContrib{indf_true}*isoContribInv{indf_true}*aperFocusImage(:,:), ...
% %         size(aperFocusImage) );
% %     toc;
% 
% 
%     %err(indf_true,:,:) =  sum(sum( abs(aperFocusImageSyn - aperFocusImage) ));
%     err(indf_true,:,:) =  sum(sum( (aperFocusImageSyn - aperFocusImage).^2, 4),1);    
% 
%     %imsyn = reshape(imflat_syn,[naper nfocus 3]);
%     %figure(89); clf; image( [im imsyn]/max(im(:)) ); pause;
% end
% toc;

% normalize results to return *per-pixel* mean squared error
%err = err/(sz(1)*sz(2)*3);  

